/* Copyright (c) 1997-2007
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
   http://www.math.tu-berlin.de/polymake,  mailto:polymake@math.tu-berlin.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
*/

#ident "$Id: barvinok_convert.h 9253 2009-05-27 14:34:21Z paffenho $"


#include <Rational.h>
#include <Integer.h>
#include <polymake.h>
#include <Matrix.h>
#include <polylib/matrix.h>

namespace polymake { namespace polytope {
    
    // Converts a vertex description in polymake format into polylib format
    // Matrix will be scaled to integral
    ::Matrix *polymake_vertices2polylib(const Matrix<Rational> &V) {

      // scale to integral coordinates: first column contains denominator
      Matrix<Rational> W = V;
      for (Entire<Rows<Matrix<Rational> > >::iterator r = entire(rows(W)); !r.at_end(); ++r ) {
	int LCM = 1;
	for ( Entire<Vector<Rational> >::const_iterator s = entire(*r); !s.at_end(); ++s ) 
	  LCM = lcm(denominator(*s),LCM);
	*r *= LCM;
      }

      // reorder columns: first will b last, set first column one (we have vertices, not rays)
      W = W|W.col(0);                               // add first column at end
      W.col(0) = ones_vector<Rational>(W.rows());   // set first column to 1

      // now put this into a polylib matrix
      int c = W.cols();
      ::Matrix *M = Matrix_Alloc(W.rows(), c);
	for (int i = 0; i < W.rows(); ++i) {
	  for (int j = 0; j < c; ++j) {
	    value_assign(M->p[i][j], mpq_numref(W[i][j].get_rep()));
	  }
	}

	return M;
    }


    // Converts an inequality description of a polytope in polymake format into polylib format
    // facet inequalities are in F, affine hull equations in A
    // both matrices must be converted to integer first
    ::Matrix *polymake_constraints2polylib(const Matrix<Integer> &F, const Matrix<Integer> &A) {
	int c = F.cols();
	::Matrix *M = Matrix_Alloc(F.rows() + A.rows(), c+1);   // polylib matrix
	for (int i = 0; i < A.rows(); ++i) {                    // assign affine hull equations
	  value_set_si(M->p[i][0], 0);
	  value_assign(M->p[i][c], A[i][0].get_rep());
	  for (int j = 1; j < c; ++j) {
	    value_assign(M->p[i][j], A[i][j].get_rep());
	  }
	}
	int j = 0;
	for (int i = A.rows(); i < F.rows()+A.rows(); ++i, ++j) {   // assign inequalities
	  value_set_si(M->p[i][0], 1);
	  value_assign(M->p[i][c], F[j][0].get_rep());
	  for (int k = 1; k < c; ++k) {
	    value_assign(M->p[i][k], F[j][k].get_rep());
	  }
	}
	return M;
    }
      
} }
